#!/bin/sh
#
# recfriio server から渡される環境変数
#
# ID # 録画設定のID
#
# SERVERURL  # webサーバ上での do-record.php までのパス ( 例 : http://localhot:80/recfriio/do-record.php )
#
# CHANNEL    # チャンネル
# DURATION   # 録画秒数
# UDP        # UDP送信する場合は 1 をセット
# IP
# PORT
#
# OUTDIR     # 保存先ディレクトリ
# NAMEBASE   # ファイル名
# FILEEXT    # 保存ファイルの拡張子
# OVERWRITE  # 0 にするとファイル名の後に日付を入れる
#
# RECFRIIO # recfriio までのフルパス
# FRIIOOPT # recfriio のオプション
#
# WGET     # wget コマンド
# WGETOPT  # wget のオプション
#
# SLEEPSEC # recfriioを起動する前のsleep秒数


# 予約録画の場合はスクリプトを起動してから sleep する
if [ "$SLEEPSEC" != 0 ];
then

# recfriio server に録画準備状態に入ったことを知らせる
GETURL="$SERVERURL?id=$ID&pid=$PID&prep=1"
$WGET $WGETOPT $GETURL > /dev/null 2>&1 &

sleep $SLEEPSEC
fi

# UDP 送信のオプション
UDPOPT=
if [ "$UDP" = 1 ];
then
UDPOPT="--udp $IP --port $PORT"
fi

# 出力ファイル名
if [ "$NAMEBASE" = "/dev/null" ];
then
  FILENAME=/dev/null
else
  # 上書きしない場合は日付を付ける
  if [ "$OVERWRITE" = 0 ];
  then
    STARTDATE=`date +%y%m%d%H%M%S`
    NAMEBASE="$NAMEBASE-$STARTDATE"
  fi
  FILENAME="$OUTDIR$NAMEBASE$FILEEXT"
fi

$RECFRIIO $FRIIOOPT $UDPOPT $CHANNEL $DURATION $FILENAME > /dev/null 2>&1 &

# PID を取得して recfriio server に録画開始を知らせる
PID=$!
GETURL="$SERVERURL?id=$ID&pid=$PID&rec=1"

if [ "$NAMEBASE" = "/dev/null" ];
then
GETURL="$GETURL&devnull=1"
fi

$WGET $WGETOPT $GETURL > /dev/null 2>&1 &

# 録画が終わるまで待つ
wait $PID

# recfriio server に録画終了を知らせる
GETURL="$SERVERURL?id=$ID&pid=$PID&stop=1"
$WGET $WGETOPT $GETURL > /dev/null 2>&1 &
