<?php

/*

    recfriioの操作関数

*/


// 起動コマンドライン作成
function create_command( $operate, &$setting )
{
    show_debug( 'create_command : operate = '.$operate );

    if( ! is_array( $setting ) || count( $setting ) != SET_SIZE ){
        throw new Exception( 'エラー(create_command) : 録画設定ファイルが壊れています。' );
    }

    // ローカルホスト
    $localhost = 'http://localhost';
    if( isset( $_SERVER['SERVER_PORT'] ) ) $localhost = $localhost.':'.$_SERVER['SERVER_PORT'];

    // do-record.phpまでのパス
    $serverpath = $_SERVER['PHP_SELF'];
    $serverpath = dirname( $serverpath ).'/do-record.php';

    // 時間
    $duration = $setting[ SET_DURATION ];
    if( $duration <= 0 ){
        throw new Exception( 'エラー(create_command) : 録画時間が正しくありません。' );
    }

    // スクリプトを起動してから sleep する時間
    $sleepsec = $setting[ SET_SLEEPSEC ];
    if( $sleepsec ) $duration += ( 60 - $sleepsec );

    // ファイル名
    if( $operate == OP_PLAY ){  // 再生のみの場合
        $outdir = '';
        $namebase = '/dev/null';
    }
    else{

        if( ! $setting[ SET_OUTDIR ] ){
            throw new Exception( 'エラー(create_command) : ディレクトリが指定されていません。' );
        }

        if( ! $setting[ SET_NAMEBASE ] ){
            throw new Exception( 'エラー(create_command) : ファイル名が指定されていません。' );
        }

        // ディレクトリの最後が'/'で無い時は追加
        $outdir = $setting[ SET_OUTDIR ];
        if( $outdir && $outdir[ strlen( $outdir ) -1 ] != '/' ) $outdir = $outdir.'/';

        $namebase = $setting[ SET_NAMEBASE ];
    }

    // UDP送信
    if( $setting[ SET_UDP ] ){

        if( ! $setting[ SET_IP ] ){
            throw new Exception( 'エラー(create_command) : IPが指定されていません。' );
            return FALSE;
        }
        if( ! $setting[ SET_PORT ] ){
            throw new Exception( 'エラー(create_command) : ポートが指定されていません。' );
        }
    }

    // 実行スクリプトの前に ./ が付いていたら取り除いてフルパスにする
    $script = get_homedir().str_replace( './', '', RECORD_SCRIPT );

    $command = 

        'ID="'.$setting[ SET_ID ].'" '.

        'SERVERURL="'.$localhost.$serverpath.'" '.

        'CHANNEL="'.$setting[ SET_CHANNEL ].'" '.
        'DURATION="'.$duration.'" '.
        'UDP="'.$setting[ SET_UDP ].'" '.
        'IP="'.$setting[ SET_IP ].'" '.
        'PORT="'.$setting[ SET_PORT ].'" '.

        'OUTDIR="'.$outdir.'" '.
        'NAMEBASE="'.$namebase.'" '.
        'FILEEXT="'.FILEEXT.'" '.
        'OVERWRITE="'.$setting[ SET_OVERWRITE ].'" '.

        'RECFRIIO="'.RECFRIIO.'" '.
        'FRIIOOPT="'.FRIIOOPT.'" '.

        'WGET="'.WGET.'" '.
        'WGETOPT="'.WGETOPT.'" '.

        'SLEEPSEC="'.$sleepsec.'" '.

        $script
        ;

    show_debug( $command );

    return $command;
}


// 起動
function exec_recfriio( $operate, &$setting )
{
    if( $operate != OP_PLAY && $operate != OP_REC ) return FALSE;

    $setting[ SET_SLEEPSEC ] = 0;
    $command = create_command( $operate, $setting );

    exec( $command.' > /dev/null 2>&1 &' );
    sleep( WAITTIME );
}


// 停止
function stop_recfriio( $pid )
{
    show_debug( 'stop_recfriio pid = '.$pid );

    $command = KILLCMD.' '.$pid;
    show_debug( $command );

    exec( $command );
    sleep( WAITTIME );
}


// crontab 登録
function regist_crontab( &$setting_array )
{
    $crontab = '';

    foreach( $setting_array as $setting ){

       if( ! is_array( $setting ) || count( $setting ) != SET_SIZE ){
           throw new Exception( 'エラー(regist_crontab) : 録画設定ファイルが壊れています。削除してください。' );
       }

       if( $setting[ SET_HOUR ] == '' || $setting[ SET_MIN ] == '' ){
           throw new Exception( 'エラー(regist_crontab) : 時刻に空白が含まれています。' );
       }

       if( ( $setting[ SET_HOUR ] < 0 || $setting[ SET_HOUR ] > 23 )
           || ( $setting[ SET_MIN ] < 0  || $setting[ SET_MIN ] > 59 ) ){
           throw new Exception( 'エラー(regist_crontab) : 時刻の範囲が間違っています。' );
       }

       $digit = 0; 
       if( $setting[ SET_SLEEPSEC ] ) $digit = -1; // 1 分前にスクリプトを起動して $setting[ SET_SLEEPSEC ] 秒 sleep する
       $min = $setting[ SET_MIN ]  + $digit;  
       if( $min < 0 ){
           $min = 59;
           $digit = -1;
       }
       else $digit = 0;
       
       $hour = $setting[ SET_HOUR ] + $digit;
       if( $hour < 0 ){
           $hour = 23;
           $digit = -1;
       }
       else $digit = 0;

       $date = $min.' '.$hour.' * * ';
       show_debug( 'date = '.$date );
       
       $week = '*';
       if( $setting[ SET_MODE ] == MODE_WEEK ){
           $week = '';
           if( $digit == 0 && $setting[ SET_SUN ] ) $week = $week.( ( 7 + $digit + 0 ) % 7 ).',';
           if( $setting[ SET_MON ] ) $week = $week.( ( 7 + $digit + 1 ) % 7 ).',';
           if( $setting[ SET_TUE ] ) $week = $week.( ( 7 + $digit + 2 ) % 7 ).',';
           if( $setting[ SET_WED ] ) $week = $week.( ( 7 + $digit + 3 ) % 7 ).',';
           if( $setting[ SET_THU ] ) $week = $week.( ( 7 + $digit + 4 ) % 7 ).',';
           if( $setting[ SET_FRI ] ) $week = $week.( ( 7 + $digit + 5 ) % 7 ).',';
           if( $setting[ SET_SAT ] ) $week = $week.( ( 7 + $digit + 6 ) % 7 ).',';
           if( $digit == -1 && $setting[ SET_SUN ] ) $week = $week.( ( 7 + $digit + 0 ) % 7 ).',';
           if( ! $week ){
               throw new Exception( 'エラー(regist_crontab) : 曜日が指定されていません。' );
           }
           $week = substr( $week, 0, strlen( $week )-1 );
       }
       show_debug( 'week = '.$week );

      if( $setting[ SET_SLEEPSEC ] <= 0 || $setting[ SET_SLEEPSEC ] >= 60  ){
           throw new Exception( 'エラー(create_command) : マージン時間が正しくありません。' );
       }

       if( $setting[ SET_ACTIVE ] ){

           $command = create_command( $operate, $setting );
           $crontab = $crontab.$date.' '.$week.' '.$command."\n";
        }
    }

    // crontab 登録
    if( ( $fp = fopen( get_vardir().CRONTABFILE, 'w' ) ) ) {

        fputs( $fp, $crontab  );
        fclose( $fp );

        exec( 'crontab '.get_vardir().CRONTABFILE );
    }
    else{
           throw new Exception( 'エラー(regist_crontab) :ファイル('.CRONTABFILE.')のオープンに失敗しました。パーミッションを確認してください。' );
    }
}


?>